<?php

class fieldMoments extends cmsFormField {

    public $title       = LANG_PARSER_MOMENTS;
    public $is_public   = true;
    public $sql         = 'text NULL DEFAULT NULL';
    public $allow_index = false;
    public $cache_sql   = 'varchar(255) NULL DEFAULT NULL';
    public $var_type    = 'array';

    public function __construct($request){

        parent::__construct($request);
        cmsCore::loadLanguage('controllers/moments/moments');

    }

    public function getItems(){
        $list = array('0'=>'');
        $ctype = $this->item['ctype_name'];
        if (!$ctype) { return $list; }
        $category_id = isset($this->item['category_id']) ? $this->item['category_id'] : 1;

        $items = cmsCore::getModel('content')->
            filterCategoryId($ctype, $category_id)->
            get('con_moments');

        if (isset($this->item['id'])){unset($items[$this->item['id']]);}

        if ($items){

            $list = $list + array_collection_to_list($items, 'id', 'title');
        }

        return $list;
    }

    public function getOptions(){

        $html = array(
            new fieldColor('color_border', array(
                'title' => LANG_MOMENTS_EVENT_COLOR_BORDER,
                'default' => '#ff0000'
            )),
            new fieldColor('color_background', array(
                'title' => LANG_MOMENTS_EVENT_COLOR_BACKGROUND,
                'default' => '#00ff00'
            )),
            new fieldColor('color_text', array(
                'title' => LANG_MOMENTS_EVENT_COLOR_TEXT,
                'default' => '#0000ff'
            )),
        );

        return $html;
    }

    public function parse($value){

        $value = is_array($value) ? $value : cmsModel::yamlToArray($value);

        return $value['mc_html'] ? $value['mc_html'] : false;
    }

    public function store($value, $is_submitted, $old_value=null){

        if (!$value['mc_start']){ return false; }

        $ctype_name = $this->item['ctype_name'];
        $id = isset($this->item['id']) ? $this->item['id'] : null;
        if ($ctype_name == 'moments'){
            $data = array(
                'item_id' => $id,
                'mc_allday' => isset($value['mc_allday']) ? $value['mc_allday'] : null,
                'mc_border_color' => $value['mc_border_color'] ? $value['mc_border_color'] : $this->getOption('color_border'),
                'mc_bg_color' => $value['mc_bg_color'] ? $value['mc_bg_color'] : $this->getOption('color_background'),
                'mc_text_color' => $value['mc_text_color'] ? $value['mc_text_color'] : $this->getOption('color_text')
            );

            $data = $data + $value;

            $moments_model = cmsCore::getModel('moments');

            if ($value['mc_parent_id'] > 0){
                $parent_time = $moments_model->getParentTime($value['mc_parent_id']);

                $new_start = new DateTime($value['mc_start']);
                $new_end = new DateTime($value['mc_end'] ? $value['mc_end'] : $value['mc_start']);
                $new_rpt_end_date = $value['mc_rpt_end_date'] ? new DateTime($value['mc_rpt_end_date']) : false;
                $new_parent_start = new DateTime($parent_time['mc_start']);
                $new_parent_end = new DateTime($parent_time['mc_end'] ? $parent_time['mc_end'] : $parent_time['mc_start']);
                $new_parent_rpt_end_date = $parent_time['mc_rpt_end_date'] ? new DateTime($parent_time['mc_rpt_end_date']) : false;

                if ($new_start < $new_parent_start){
                    $value['mc_start'] = $new_parent_start->format('Y-m-d H:i');
                }

                if ($new_start > $new_parent_end){
                    $value['mc_start'] = $new_parent_end->format('Y-m-d H:i');
                }

                if ($new_end > $new_parent_end){
                    $value['mc_end'] = $new_parent_end->format('Y-m-d H:i');
                }

                if ($new_end < $new_parent_start){
                    $value['mc_end'] = $new_start > $new_end ? $new_start->format('Y-m-d H:i') : $new_end->format('Y-m-d H:i');
                }

                if ($new_parent_rpt_end_date){
                    if (!$new_rpt_end_date  || ($new_rpt_end_date > $new_parent_rpt_end_date)){
                        $value['mc_rpt_end_date'] = $new_parent_rpt_end_date->format('Y-m-d H:i');
                        $data['mc_rpt_end_date'] = $value['mc_rpt_end_date'];
                    }
                }
                $data['mc_start'] = $value['mc_start'];
                $data['mc_end'] = $value['mc_end'];
            }

            $moments_model->addOrUpdateMoment($data);
        }

        $html = '';
        $html .= LANG_PARSER_MOMENTS_START_IN . ': ' . $value['mc_start'] . '.' . PHP_EOL;
        $html .= $value['mc_end'] && $value['mc_end'] != $value['mc_start'] ? LANG_PARSER_MOMENTS_END_IN . ': ' . $value['mc_end'] . '.' . PHP_EOL : '';

        if (isset($value['mc_allday'])){
            $diff_str = LANG_PARSER_MOMENTS_EVENT_ALLDAY;

        } elseif (isset($value['mc_end'])) {
            $length = real_date_diff(date('Y-m-d H:i:s', strtotime($value['mc_start'])), date('Y-m-d H:i:s', strtotime($value['mc_end'])));

            $diff_str = array();

            if ($length[0]){
                $diff_str[] = html_spellcount($length[0], LANG_YEAR1, LANG_YEAR2, LANG_YEAR10);
            }
            if ($length[1]){
                $diff_str[] = html_spellcount($length[1], LANG_MONTH1, LANG_MONTH2, LANG_MONTH10);
            }
            if ($length[2]){
                $diff_str[] = html_spellcount($length[2], LANG_DAY1, LANG_DAY2, LANG_DAY10);
            }
            if ($length[3]){
                $diff_str[] = html_spellcount($length[3], LANG_HOUR1, LANG_HOUR2, LANG_HOUR10);
            }
            if ($length[4]){
                $diff_str[] = html_spellcount($length[4], LANG_MINUTE1, LANG_MINUTE2, LANG_MINUTE10);
            }

            $diff_str = trim( implode(' ', $diff_str) );
        } else {
            $diff_str = LANG_PARSER_MOMENTS_NO_LENGTH;
        }
        $html .= LANG_PARSER_MOMENTS_EVENT_LENGTH . ': ' . $diff_str . '.' . PHP_EOL;

        $value['mc_html'] = htmlspecialchars(trim($html));

        return $value;

    }

    public function getInput($value){

        $this->data = false;

        if($value){
            $this->data = is_array($value) ? $value : cmsModel::yamlToArray($value);
        }

        return parent::getInput($value);
    }
}